@@ -16,7 +16,7 @@
 # All available configuration variables are documente at
 # http://doc.scrapy.org/en/latest/topics/settings.html.
 
-from scrapy import linkextractor
+from scrapy import linkextractors
 
 BOT_NAME = 'sitemap'
 SPIDER_MODULES = ['generator.spiders']
@@ -27,8 +27,9 @@ ITEM_PIPELINES = {
 CONCURRENT_REQUESTS = 32
 CONCURRENT_REQUESTS_PER_DOMAIN = 32
 CONCURRENT_REQUESTS_PER_IP = 32
+DOWNLOAD_WARNSIZE = 67108864
 LOG_LEVEL = 'INFO'
 LOGGING_ENABLED = True
 RANDOMIZE_DOWNLOAD_DELAY = False
 TELNETCONSOLE_ENABLED = False
-linkextractor.IGNORED_EXTENSIONS.remove('pdf')
+linkextractors.IGNORED_EXTENSIONS.remove('pdf')
